#vider la mmoire
rm(list=ls())

#importer l'chantillon d'apprentissage
#dans la premire feuille
library(xlsx)
DTrain <- read.xlsx("waveform.xlsx",sheetIndex=1)

#description des donnes
str(DTrain)

#stat. descriptives
print(summary(DTrain))

#projeter dans le plan par paire
#on note que les diffrentes classes se distinguent bien
#selon les combinaisons de variables
pairs(DTrain[-ncol(DTrain)],col=DTrain$classe)

#centrer et rduire les explicatives
ZTrain <- scale(DTrain[-ncol(DTrain)],center=TRUE,scale=TRUE)

#summary des donnes
print(summary(ZTrain))

#info. supplmentaires
print(attributes(ZTrain))

#accs  l'attribut scaled:center
#reprsentant les moyennes utilises
print(attr(ZTrain,'scaled:center'))

#et les carts-type
print(attr(ZTrain,'scaled:scale'))

#librairie nnet
library(nnet)

#modliser
#il faut travailler sur les donnes standardises parce que
#l'heuristique d'optimisation peut tre gn par les diffrences
#d'chelle entre les variables
#nous obtenons les coefficients estims, la dviance, le critre AIC
mNet <- nnet::multinom(classe ~ ., data = cbind(ZTrain,DTrain['classe']))
print(mNet)

#attributs de l'objet
print(attributes(mNet))

#summary
sNet <- summary(mNet)
print(sNet)

#attributs
print(attributes(sNet))

#coefficients
print(sNet$coefficients)

#type d'objet
print(class(sNet$coefficients))

#probabilits approches
head(sNet$fitted.values)

#dimensions
print(dim(sNet$fitted.values))

#vrifier que la some par ligne vaut bien 1
print(rowSums(sNet$fitted.values))

#vrifions pour le premier individu
#cours -- page 5

#modalit B
eB <- exp(sum(c(1,ZTrain[1,]) * sNet$coefficients["B",]))
print(eB)

#modalit C
eC <- exp(sum(c(1,ZTrain[1,]) * sNet$coefficients["C",]))
print(eC)

#proba B
pB <- eB/(1.0 + (eB + eC))
print(pB)

#proba C
pC <- eC/(1.0 + (eB + eC))
print(pC)

#proba 
pA <- 1.0 - (pB + pC)
print(pA)

#vrifier la pertinence globale du modle
#via le test du rapport de vraisemblance

#null modle
mNull <- nnet::multinom(classe ~ 1, data = DTrain)
print(mNull)

#test du rapport de vraisemblance
LRGlob <- mNull$deviance - mNet$deviance
print(LRGlob)

#degrs de libert
ddlGlob <- mNet$edf - mNull$edf
print(ddlGlob)

#p-value
print(pchisq(LRGlob,ddlGlob,lower.tail=FALSE))

#vrifier la pertinence de chaque variable
#via le test du rapport de vraisemblance de nullit des coefs. associs
for (j in 1:ncol(ZTrain)){
  #rgression sans la variable  tester
  mTemp <- nnet::multinom(classe ~ ., data = cbind(ZTrain[,-j],DTrain['classe']))
  #Rapport de vraisemblance
  LR <- mTemp$deviance - mNet$deviance
  #degr de libert
  ddl <- mNet$edf - mTemp$edf
  #p-value
  pval <- pchisq(LR,ddl,lower.tail=FALSE)
  #affichage
  print(paste(colnames(ZTrain)[j],": "))
  print(c(LR,ddl,pval))
}

#charger l'chantillon test
DTest <- read.xlsx("waveform.xlsx",sheetIndex=2)

#structure
str(DTest)

#rsum stat.
print(summary(DTest))

#centrer et rduire
#/!\ avec les param. (moyennes, carts-type) 
#calculs sur l'chantillon d'apprentissage
#parce que les individus de l'chantillon test reprsentent la population de dploiement
#ils doivent tre traits individuellement et non pas collectivement
#aucun paramtre ne doit tre calcul sur cet chantillon
ZTest <- scale(DTest[-ncol(DTest)],center=attr(ZTrain,'scaled:center'),scale=attr(ZTrain,'scaled:scale'))
print(summary(ZTest))

#prdiction
predNet <- predict(mNet,newdata=ZTest,type="class")

#distribution des classes prdites
print(table(predNet))

#construire la matrice de confusion
mcNet <- table(DTest$classe,predNet)
print(mcNet)

#calculer le taux de reconnaissance  partir de la matrice de confusion
accNet <- sum(diag(mcNet))/sum(mcNet)
print(accNet)

#calculer le taux d'erreur  partir de la matrice de confusion
errNet <- 1 - accNet
print(errNet)

#calculer les rappels par classe
rappelNet <- diag(mcNet)/rowSums(mcNet)
print(rappelNet)

#calculer les prcisions par classe
prNet <- diag(mcNet)/colSums(mcNet)
print(prNet)

#vrificatin avec la librairie "caret"
library(caret)

#confrontation des rsultats -- ok
print(caret::confusionMatrix(data=predNet,reference=DTest$classe))

#chargement de la librairie
library(VGAM)

#modlisation
mVg <- VGAM::vglm(classe ~ ., data = cbind(ZTrain,DTrain['classe']), family=multinomial(refLevel = "A"))
print(mVg)

#affichage dtaill
#on les tests de significativit des coefs.
#mais par quation, trs peu exploitable
sVg <- summary(mVg)
print(sVg)

#prediction -- response est celui qui nous intresse le plus
predVglm <- predictvglm(object=mVg,newdata=as.data.frame(ZTest),type="response")

#premires lignes
print(head(predVglm))

#prdiction des classes
predClassVglm <- factor(c("A","B","C")[apply(predVglm,1,which.max)])

#dist. des prdictions
print(table(predClassVglm))

#confrontation avec la classe observe
print(caret::confusionMatrix(data=predClassVglm,reference=DTest$classe))

#arbre de dcision avec rpart
library(rpart)

#on peut travailler directement avec les variables originelles
#le traitement des donnes centres et rduites donnerait exactement le mme rsultat
mArbre <- rpart(classe ~ ., data = DTrain)
print(mArbre)

#affichage graphique
library(rpart.plot)
rpart.plot(mArbre)

#liste des attributs des l'arbre
print(attributes(mArbre))

#structure de l'arbre
print(mArbre$frame)

#liste des variables prises en compte dans l'arbre
#il n'y a pas V07, qui est pourtant significative dans la rgression
print(setdiff(unique(mArbre$frame$'var'),c('<leaf>')))

#dtail et variables comptitives dans l'arbre sur chaque noeud
#voir en particulier : << Primary Splits >>
#sur la racine par exemple, on voit que V07 tait en 3e position
#dans la comptition pour segmenter le sommet
print(summary(mArbre))

#prdiction
predArbre <- predict(mArbre,newdata=DTest,type="class")
print(table(predArbre))

#performances - nettement moins bon que la rgression logistique
print(caret::confusionMatrix(data=predArbre,reference=DTest$classe))
